@@ -1,12 +1,11 @@
 """Tutorial: DDFM for Finance Data
 
-This tutorial demonstrates the complete workflow for training, prediction, and nowcasting
+This tutorial demonstrates the complete workflow for training, prediction
 using finance data with market_forward_excess_returns as the target variable.
 
 Target: market_forward_excess_returns
 Excluded: risk_free_rate, forward_returns
 
-Nowcasting Pattern: model.update(X_std).predict(horizon=1)
 """
 
 import sys
@@ -20,8 +19,9 @@ import pandas as pd
 import numpy as np
 from datetime import datetime
 from dfm_python import DDFM, DDFMDataModule, DDFMTrainer
-from dfm_python.config import DFMConfig, SeriesConfig, DEFAULT_BLOCK_NAME
-from dfm_python.utils.time import TimeIndex, parse_timestamp
+from dfm_python.config import DDFMConfig
+from dfm_python.utils.misc import TimeIndex
+from dfm_python.dataset.process import parse_timestamp
 
 # sktime imports for preprocessing
 from sktime.transformations.compose import TransformerPipeline
@@ -136,35 +136,26 @@ df_processed = df_preprocessed
 # ============================================================================
 print("\n[Step 3] Creating configuration...")
 
-# Create series configs
-series_configs = []
-for col in selected_cols:
-    # SeriesConfig only needs frequency (transformation and blocks removed)
-    series_configs.append(
-        SeriesConfig(
-            series_id=col,
-            frequency="m"  # Assuming monthly
-        )
-    )
+# Create frequency dict (maps column names to frequencies)
+# All series are monthly, so use 'm' for all
+frequency_dict = {col: "m" for col in selected_cols}
 
-# Create blocks config - VAR(1) only
-blocks_config = {
-    DEFAULT_BLOCK_NAME: {
-        "factors": 1,  # Reduced to 1 for faster execution
-        "ar_lag": 1,   # VAR(1) - first-order autoregressive
-        "clock": "m"
-    }
-}
-
-# Create DFM config
-config = DFMConfig(
-    series=series_configs,
-    blocks=blocks_config
+# Create DDFM config (DDFM does not use blocks structure)
+# DDFM uses num_factors directly, not blocks
+config = DDFMConfig(
+    frequency=frequency_dict,
+    clock="m",  # Monthly clock frequency
+    num_factors=1,  # Reduced to 1 for faster execution
+    factor_order=1,  # VAR(1) - first-order autoregressive
+    encoder_layers=[32, 16],  # Reduced for faster execution
+    epochs=10,  # Reduced for faster execution
+    learning_rate=0.001,
+    batch_size=32
 )
 
-print(f"   Number of series: {len(series_configs)}")
-print(f"   Number of factors: 1 (DDFM uses num_factors parameter)")
-print(f"   Factor dynamics: VAR(1) (ar_lag=1)")
+print(f"   Number of series: {len(selected_cols)}")
+print(f"   Number of factors: {config.num_factors} (DDFM uses num_factors parameter)")
+print(f"   Factor dynamics: VAR(1) (factor_order=1)")
 print(f"   Target series: {target_col}")
 
 # ============================================================================
@@ -234,7 +225,7 @@ X_forecast_history = None
 Z_forecast_history = None
 
 try:
-    # Predict with default horizon (target=None uses DataModule's target_series)
+    # Predict with default horizon (uses target_series from DataModule)
     X_forecast, Z_forecast = model.predict(horizon=6)
     
     print(f"   Forecast shape: {X_forecast.shape} (target series only)")
@@ -246,13 +237,17 @@ try:
         print(f"   First forecast values (targets): {X_forecast[0, :]}")
     
     # Predict with history parameter (using recent 60 periods)
-    X_forecast_history, Z_forecast_history = model.predict(horizon=6, history=60)
+    # Note: history parameter was removed - prediction uses full history by default
     
-    print(f"   Forecast with history shape: {X_forecast_history.shape} (target series only)")
-    if X_forecast_history.shape[1] == 1:
-        print(f"   First forecast with history (target): {X_forecast_history[0, 0]:.6f}")
+    # Note: history parameter was removed - prediction uses full history by default
+    if X_forecast_history is not None:
+        print(f"   Forecast with history shape: {X_forecast_history.shape} (target series only)")
+        if X_forecast_history.shape[1] == 1:
+            print(f"   First forecast with history (target): {X_forecast_history[0, 0]:.6f}")
+        else:
+            print(f"   First forecast with history (targets): {X_forecast_history[0, :]}")
     else:
-        print(f"   First forecast with history (targets): {X_forecast_history[0, :]}")
+        print(f"   Forecast with history: Not available (history parameter was removed)")
     
 except ValueError as e:
     print(f"   Prediction failed: {e}")
@@ -262,60 +257,7 @@ except ValueError as e:
     print("   - Using different factor configurations")
 
 # ============================================================================
-# Step 7: Nowcasting with update().predict() pattern
-# ============================================================================
-print("\n[Step 7] Nowcasting using update().predict() pattern...")
-
-try:
-    # Get the trained model's result for standardization parameters
-    result = model.result
-    Mx = result.Mx  # Mean for standardization
-    Wx = result.Wx  # Standard deviation for standardization
-    
-    # Simulate new data for nowcasting (in practice, this would be real-time data)
-    # Use the last few periods of training data as "new" data
-    n_new_periods = 5
-    X_new_raw = df_processed.iloc[-n_new_periods:].values
-    
-    # Standardize new data using the same parameters from training
-    # Standardization: (X - Mx) / Wx
-    X_new_std = (X_new_raw - Mx) / Wx
-    
-    # Handle any NaN values (missing data in new observations)
-    X_new_std = np.where(np.isfinite(X_new_std), X_new_std, np.nan)
-    
-    print(f"   New data shape: {X_new_std.shape}")
-    print(f"   Standardized new data (first row): {X_new_std[0, :5]}")
-    
-    # Update model state with new standardized data, then predict
-    # Pattern: model.update(X_std).predict(horizon=1)
-    # predict() returns only target series (target=None uses DataModule's target_series)
-    X_nowcast, Z_nowcast = model.update(X_new_std).predict(horizon=1)
-    
-    # X_nowcast now contains only target series (no features)
-    if X_nowcast.shape[1] == 1:
-        nowcast_value = X_nowcast[0, 0]
-    else:
-        nowcast_value = X_nowcast[0, :]
-    
-    print(f"   Nowcast value for {target_col}: {nowcast_value:.6f}")
-    print(f"   Nowcast uses VAR(1) factor dynamics")
-    
-    # Alternative: Update and predict separately
-    model.update(X_new_std)
-    X_nowcast2, Z_nowcast2 = model.predict(horizon=1)
-    if X_nowcast2.shape[1] == 1:
-        nowcast_value2 = X_nowcast2[0, 0]
-    else:
-        nowcast_value2 = X_nowcast2[0, :]
-    print(f"   Alternative pattern (separate calls): {nowcast_value2:.6f}")
-    
-except (ValueError, AttributeError, IndexError) as e:
-    print(f"   Nowcasting failed: {e}")
-    print("   Note: Ensure model is trained and data is properly standardized")
-
-# ============================================================================
-# Step 8: Summary
+# Step 7: Summary
 # ============================================================================
 print("\n" + "=" * 80)
 print("Tutorial Summary")
@@ -326,6 +268,5 @@ if X_forecast is not None:
     print(f"✅ Predictions generated: {X_forecast.shape[0]} periods ahead")
 else:
     print(f"⚠️  Predictions: Failed (see error message above)")
-print(f"✅ Nowcasting pattern: model.update(X_std).predict(horizon=1)")
 print(f"✅ Target series: {target_col}")
 print("=" * 80)
