@@ -1,12 +1,10 @@
 """Tutorial: DFM for Finance Data
 
-This tutorial demonstrates the complete workflow for training, prediction, and nowcasting
+This tutorial demonstrates the complete workflow for training and prediction
 using finance data with market_forward_excess_returns as the target variable.
 
 Target: market_forward_excess_returns
 Excluded: risk_free_rate, forward_returns
-
-Nowcasting Pattern: model.update(X_std).predict(horizon=1)
 """
 
 import sys
@@ -19,9 +17,10 @@ sys.path.insert(0, str(project_root / "src"))
 import pandas as pd
 import numpy as np
 from datetime import datetime
-from dfm_python import DFM, DFMDataModule, DFMTrainer
-from dfm_python.config import DFMConfig, SeriesConfig, DEFAULT_BLOCK_NAME
-from dfm_python.utils.time import TimeIndex, parse_timestamp
+from dfm_python import DFM, DFMDataModule
+from dfm_python.config import DFMConfig
+from dfm_python.utils.misc import TimeIndex
+from dfm_python.dataset.process import parse_timestamp
 
 # sktime imports for preprocessing
 from sktime.transformations.compose import TransformerPipeline
@@ -134,49 +133,31 @@ df_processed = df_preprocessed
 # ============================================================================
 print("\n[Step 3] Creating configuration...")
 
-# Create series configs
-series_configs = []
-for col in selected_cols:
-    if col == target_col:
-        # Target series
-        series_configs.append(
-            SeriesConfig(
-                series_id=col,
-                frequency="m",  # Assuming monthly
-                transformation="lin",
-                blocks=[DEFAULT_BLOCK_NAME]
-            )
-        )
-    else:
-        # Predictor series
-        series_configs.append(
-            SeriesConfig(
-                series_id=col,
-                frequency="m",
-                transformation="lin",
-                blocks=[DEFAULT_BLOCK_NAME]
-            )
-        )
+# Create frequency dict (maps column names to frequencies)
+# All series are monthly, so use 'm' for all
+frequency_dict = {col: "m" for col in selected_cols}
 
-# Create blocks config - VAR(1) only
+# Create blocks config - new format: {"block_name": {"num_factors": int, "series": [str]}}
+# VAR(1) is specified globally via ar_lag parameter
 blocks_config = {
-    DEFAULT_BLOCK_NAME: {
-        "factors": 1,  # Reduced to 1 for faster execution
-        "ar_lag": 1,   # VAR(1) - first-order autoregressive
-        "clock": "m"
+    "Block_Global": {
+        "num_factors": 1,  # Reduced to 1 for faster execution
+        "series": selected_cols  # All series in one block
     }
 }
 
 # Create DFM config
 config = DFMConfig(
-    series=series_configs,
+    frequency=frequency_dict,
     blocks=blocks_config,
+    clock="m",  # Monthly clock frequency
+    ar_lag=1,   # VAR(1) - first-order autoregressive (global parameter)
     max_iter=3,  # Further reduced for faster execution
     threshold=1e-2  # More relaxed threshold for faster convergence
 )
 
-print(f"   Number of series: {len(series_configs)}")
-print(f"   Number of factors: {config.blocks[DEFAULT_BLOCK_NAME]['factors']}")
+print(f"   Number of series: {len(selected_cols)}")
+print(f"   Number of factors: {config.blocks['Block_Global']['num_factors']}")
 print(f"   Factor dynamics: VAR(1) (ar_lag=1)")
 print(f"   Target series: {target_col}")
 
@@ -225,12 +206,21 @@ print("\n[Step 5] Training DFM model...")
 # Note: mixed_freq=False (default) since all series are monthly (unified frequency)
 # Set mixed_freq=True if you have mixed frequencies (e.g., quarterly + monthly)
 model = DFM(mixed_freq=False)
-model._config = config  # Set config directly
+model.load_config(config)  # Load config
 
-trainer = DFMTrainer(max_epochs=1)  # Minimal epochs for faster execution
-trainer.fit(model, data_module)
+# Get initialization parameters from datamodule
+init_params = data_module.get_initialization_params()
+X = init_params['X']
+Mx = init_params['Mx']
+Wx = init_params['Wx']
+
+# Fit model directly (DFM uses fit() method, not Lightning trainer)
+training_state = model.fit(X=X, Mx=Mx, Wx=Wx, datamodule=data_module)
 
 print("   Training completed!")
+print(f"   Converged: {training_state.converged}")
+print(f"   Iterations: {training_state.num_iter}")
+print(f"   Log-likelihood: {training_state.loglik:.4f}")
 
 # ============================================================================
 # Step 6: Prediction
@@ -243,18 +233,14 @@ X_forecast_history = None
 Z_forecast_history = None
 
 try:
-    # Predict with default horizon
+    # Predict with default horizon, specifying target series
     X_forecast, Z_forecast = model.predict(horizon=6)
     
     print(f"   Forecast shape: {X_forecast.shape}")
     print(f"   Factor forecast shape: {Z_forecast.shape}")
-    print(f"   First forecast values (target): {X_forecast[0, -1]:.6f}")
+    print(f"   First forecast value (target): {X_forecast[0, 0]:.6f}")
     
-    # Predict with history parameter (using recent 60 periods)
-    X_forecast_history, Z_forecast_history = model.predict(horizon=6, history=60)
-    
-    print(f"   Forecast with history shape: {X_forecast_history.shape}")
-    print(f"   First forecast with history (target): {X_forecast_history[0, -1]:.6f}")
+    # Note: history parameter was removed - prediction uses full history by default
     
 except ValueError as e:
     print(f"   Prediction failed: {e}")
@@ -264,64 +250,16 @@ except ValueError as e:
     print("   - Using different factor configurations")
 
 # ============================================================================
-# Step 7: Nowcasting with update().predict() pattern
-# ============================================================================
-print("\n[Step 7] Nowcasting using update().predict() pattern...")
-
-try:
-    # Get the trained model's result for standardization parameters
-    result = model.result
-    Mx = result.Mx  # Mean for standardization
-    Wx = result.Wx  # Standard deviation for standardization
-    
-    # Simulate new data for nowcasting (in practice, this would be real-time data)
-    # Use the last few periods of training data as "new" data
-    n_new_periods = 5
-    X_new_raw = df_processed.iloc[-n_new_periods:].values
-    
-    # Standardize new data using the same parameters from training
-    # Standardization: (X - Mx) / Wx
-    X_new_std = (X_new_raw - Mx) / Wx
-    
-    # Handle any NaN values (missing data in new observations)
-    X_new_std = np.where(np.isfinite(X_new_std), X_new_std, np.nan)
-    
-    print(f"   New data shape: {X_new_std.shape}")
-    print(f"   Standardized new data (first row): {X_new_std[0, :5]}")
-    
-    # Update model state with new standardized data, then predict
-    # Pattern: model.update(X_std).predict(horizon=1)
-    X_nowcast, Z_nowcast = model.update(X_new_std).predict(horizon=1)
-    
-    # Extract nowcast for target series
-    target_idx = selected_cols.index(target_col)
-    nowcast_value = X_nowcast[0, target_idx]
-    
-    print(f"   Nowcast value for {target_col}: {nowcast_value:.6f}")
-    print(f"   Nowcast uses VAR(1) factor dynamics")
-    
-    # Alternative: Update and predict separately
-    model.update(X_new_std)
-    X_nowcast2, Z_nowcast2 = model.predict(horizon=1)
-    nowcast_value2 = X_nowcast2[0, target_idx]
-    print(f"   Alternative pattern (separate calls): {nowcast_value2:.6f}")
-    
-except (ValueError, AttributeError, IndexError) as e:
-    print(f"   Nowcasting failed: {e}")
-    print("   Note: Ensure model is trained and data is properly standardized")
-
-# ============================================================================
-# Step 8: Summary
+# Step 7: Summary
 # ============================================================================
 print("\n" + "=" * 80)
 print("Tutorial Summary")
 print("=" * 80)
 print(f"✅ Data loaded: {df.shape[0]} rows, {len(selected_cols)} series")
-print(f"✅ Model trained: {len(series_configs)} series, {config.blocks[DEFAULT_BLOCK_NAME]['factors']} factors, VAR(1) dynamics")
+print(f"✅ Model trained: {len(selected_cols)} series, {config.blocks['Block_Global']['num_factors']} factors, VAR(1) dynamics")
 if X_forecast is not None:
     print(f"✅ Predictions generated: {X_forecast.shape[0]} periods ahead")
 else:
     print(f"⚠️  Predictions: Failed (see error message above)")
-print(f"✅ Nowcasting pattern: model.update(X_std).predict(horizon=1)")
 print(f"✅ Target series: {target_col}")
 print("=" * 80)
