@@ -1,11 +1,9 @@
 """Tutorial: DFM for Macro Data
 
-This tutorial demonstrates the complete workflow for training, prediction, and nowcasting
-using macro data with KOEQUIPTE as the target variable.
+This tutorial demonstrates the complete workflow for training and prediction
+using macro data with multiple target variables.
 
-Target: KOEQUIPTE (Investment, Equipment, Estimation, SA)
-
-Nowcasting Pattern: model.update(X_std).predict(horizon=1)
+Targets: KOEQUIPTE (Investment), KOWRCCNSE (Consumption), KOIMPCONA (Imports)
 """
 
 import sys
@@ -18,9 +16,9 @@ sys.path.insert(0, str(project_root / "src"))
 import pandas as pd
 import numpy as np
 from datetime import datetime
-from dfm_python import DFM, DFMDataModule, DFMTrainer
-from dfm_python.config import DFMConfig, SeriesConfig, DEFAULT_BLOCK_NAME
-from dfm_python.utils.time import TimeIndex
+from dfm_python import DFM, DFMDataModule
+from dfm_python.config import DFMConfig
+from dfm_python.utils.misc import TimeIndex
 
 # sktime imports for preprocessing
 from sktime.transformations.compose import TransformerPipeline
@@ -46,8 +44,8 @@ print(f"   Columns: {len(df.columns)}")
 # ============================================================================
 print("\n[Step 2] Preparing data...")
 
-# Target variable
-target_col = "KOEQUIPTE"
+# Target variables (multiple targets)
+target_cols = ["KOEQUIPTE", "KOWRCCNSE", "KOIMPCONA"]  # Investment, Consumption, Imports
 
 # Select a subset of series for faster execution
 # Use fewer series for faster execution
@@ -60,9 +58,8 @@ selected_cols = [
     "KOIMPCONA",
     # Production (reduced)
     "KOCONPRCF",
-    # Target
-    target_col
-]
+    # Targets (included in selected_cols)
+] + target_cols
 
 # Filter to only columns that exist in the data
 selected_cols = [col for col in selected_cols if col in df.columns]
@@ -101,13 +98,13 @@ print("\n[Step 2.5] Creating preprocessing pipeline with sktime...")
 # This is faster than ColumnEnsembleTransformer for small datasets
 
 # Apply difference transformation to target series manually (for chg transformation)
-if target_col in df_processed.columns:
-    target_idx = df_processed.columns.get_loc(target_col)
-    target_series = df_processed[target_col].values
-    # Apply first difference
-    target_diff = np.diff(target_series, prepend=target_series[0])
-    df_processed[target_col] = target_diff
-    print(f"   Applied difference transformation to {target_col}")
+for target_col in target_cols:
+    if target_col in df_processed.columns:
+        target_series = df_processed[target_col].values
+        # Apply first difference
+        target_diff = np.diff(target_series, prepend=target_series[0])
+        df_processed[target_col] = target_diff
+        print(f"   Applied difference transformation to {target_col}")
 
 # Note: date column will be removed by DataModule when time_index_column='date' is used
 # For preprocessing, we need to temporarily remove it to avoid issues with datetime columns
@@ -132,6 +129,11 @@ preprocessing_pipeline = TransformerPipeline(
 
 def _get_fitted_scaler(pipeline, data_frame):
     """Extract fitted scaler; if not fitted, fit it on provided data."""
+    # Drop non-numeric columns before fitting
+    numeric_cols = data_frame.select_dtypes(include=[np.number]).columns
+    if len(numeric_cols) < len(data_frame.columns):
+        data_frame = data_frame[numeric_cols].copy()
+    
     steps_attr = getattr(pipeline, "steps_", None)
     candidate = (steps_attr or pipeline.steps)[-1][1]
     if not hasattr(candidate, "n_features_in_"):
@@ -139,7 +141,7 @@ def _get_fitted_scaler(pipeline, data_frame):
     return candidate
 
 print("   Pipeline: Imputer(ffill) → Imputer(bfill) → StandardScaler")
-print(f"   Transformations: {target_col} uses difference (chg), others use linear")
+print(f"   Transformations: {', '.join(target_cols)} use difference (chg), others use linear")
 print("   Applying preprocessing pipeline...")
 
 # Fit preprocessing pipeline on training data once (without date column)
@@ -183,51 +185,33 @@ df_processed = df_preprocessed
 # ============================================================================
 print("\n[Step 3] Creating configuration...")
 
-# Create series configs
-series_configs = []
-for col in selected_cols:
-    if col == target_col:
-        # Target series - use chg transformation (as per series config)
-        series_configs.append(
-            SeriesConfig(
-                series_id=col,
-                frequency="m",
-                transformation="chg",  # As per KOEQUIPTE.yaml
-                blocks=[DEFAULT_BLOCK_NAME]
-            )
-        )
-    else:
-        # Predictor series - use lin for simplicity
-        series_configs.append(
-            SeriesConfig(
-                series_id=col,
-                frequency="m",
-                transformation="lin",
-                blocks=[DEFAULT_BLOCK_NAME]
-            )
-        )
+# Create frequency dict (maps column names to frequencies)
+# All series are monthly, so use 'm' for all
+frequency_dict = {col: "m" for col in selected_cols}
 
-# Create blocks config - VAR(1) only
+# Create blocks config - new format: {"block_name": {"num_factors": int, "series": [str]}}
+# VAR(1) is specified globally via ar_lag parameter
 blocks_config = {
-    DEFAULT_BLOCK_NAME: {
-        "factors": 1,  # Reduced to 1 for faster execution
-        "ar_lag": 1,   # VAR(1) - first-order autoregressive
-        "clock": "m"
+    "Block_Global": {
+        "num_factors": 1,  # Reduced to 1 for faster execution
+        "series": selected_cols  # All series in one block
     }
 }
 
 # Create DFM config
 config = DFMConfig(
-    series=series_configs,
+    frequency=frequency_dict,
     blocks=blocks_config,
+    clock="m",  # Monthly clock frequency
+    ar_lag=1,   # VAR(1) - first-order autoregressive (global parameter)
     max_iter=3,  # Further reduced for faster execution
     threshold=1e-2  # More relaxed threshold for faster convergence
 )
 
-print(f"   Number of series: {len(series_configs)}")
-print(f"   Number of factors: {config.blocks[DEFAULT_BLOCK_NAME]['factors']}")
+print(f"   Number of series: {len(selected_cols)}")
+print(f"   Number of factors: {config.blocks['Block_Global']['num_factors']}")
 print(f"   Factor dynamics: VAR(1) (ar_lag=1)")
-print(f"   Target series: {target_col}")
+print(f"   Target series: {', '.join(target_cols)} ({len(target_cols)} targets)")
 
 # ============================================================================
 # Step 4: Create DataModule
@@ -243,7 +227,8 @@ data_module = DFMDataModule(
     data=df_processed,  # Pass DataFrame directly (not .values)
     time_index_column='date',  # Extract time index from 'date' column and exclude it from data
     pipeline=fitted_pipeline,  # Already fitted pipeline
-    preprocessed=True  # Data is already preprocessed
+    preprocessed=True,  # Data is already preprocessed
+    target_series=target_cols  # Specify multiple target series
 )
 data_module.setup()
 
@@ -261,15 +246,24 @@ if data_module.time_index is not None:
 print("\n[Step 5] Training DFM model...")
 
 # Create DFM model
-# Note: mixed_freq=False (default) since all series are monthly (unified frequency)
-# Set mixed_freq=True if you have mixed frequencies (e.g., quarterly + monthly)
-model = DFM(mixed_freq=False)
-model._config = config  # Set config directly
+# Note: mixed_freq is now auto-detected from DataModule
+# Mixed frequency will be automatically detected based on config frequencies
+model = DFM()
+model.load_config(config)  # Load config
 
-trainer = DFMTrainer(max_epochs=1)  # Minimal epochs for faster execution
-trainer.fit(model, data_module)
+# Get initialization parameters from datamodule
+init_params = data_module.get_initialization_params()
+X = init_params['X']
+Mx = init_params['Mx']
+Wx = init_params['Wx']
+
+# Fit model directly (DFM uses fit() method, not Lightning trainer)
+training_state = model.fit(X=X, Mx=Mx, Wx=Wx, datamodule=data_module)
 
 print("   Training completed!")
+print(f"   Converged: {training_state.converged}")
+print(f"   Iterations: {training_state.num_iter}")
+print(f"   Log-likelihood: {training_state.loglik:.4f}")
 
 # ============================================================================
 # Step 6: Prediction
@@ -284,15 +278,17 @@ Z_forecast_history = None
 scaler = _get_fitted_scaler(fitted_pipeline, df_for_preprocessing)
 
 try:
-    # Predict with default horizon
+    # Predict with default horizon (uses target_series from DataModule)
     X_forecast, Z_forecast = model.predict(horizon=6)
     
     print(f"   Forecast shape: {X_forecast.shape}")
     print(f"   Factor forecast shape: {Z_forecast.shape}")
     
-    # Find target index
-    target_idx = selected_cols.index(target_col)
-    print(f"   First forecast value (target {target_col}): {X_forecast[0, target_idx]:.6f}")
+    # X_forecast contains forecasts for all target series
+    print(f"   Forecast shape: {X_forecast.shape} (horizon x {len(target_cols)} targets)")
+    print(f"   First period forecasts:")
+    for i, target_col in enumerate(target_cols):
+        print(f"     {target_col}: {X_forecast[0, i]:.6f}")
     
     # Verify inverse-transform consistency (round-trip through scaler)
     try:
@@ -302,17 +298,7 @@ try:
     except Exception as inv_err:
         print(f"   ⚠ Inverse-transform check failed (predict): {inv_err}")
     
-    # Predict with history parameter (using recent 60 periods)
-    X_forecast_history, Z_forecast_history = model.predict(horizon=6, history=60)
-    
-    print(f"   Forecast with history shape: {X_forecast_history.shape}")
-    print(f"   First forecast with history (target): {X_forecast_history[0, target_idx]:.6f}")
-    try:
-        restored_hist = scaler.inverse_transform(scaler.transform(X_forecast_history))
-        assert np.allclose(restored_hist, X_forecast_history, atol=1e-6)
-        print("   ✔ Inverse-transform check passed (predict with history)")
-    except Exception as inv_err:
-        print(f"   ⚠ Inverse-transform check failed (history): {inv_err}")
+    # Note: history parameter was removed - prediction uses full history by default
     
 except ValueError as e:
     print(f"   Prediction failed: {e}")
@@ -322,93 +308,16 @@ except ValueError as e:
     print("   - Using different factor configurations")
 
 # ============================================================================
-# Step 7: Nowcasting with update().predict() pattern
-# ============================================================================
-print("\n[Step 7] Nowcasting using update().predict() pattern...")
-
-try:
-    # Get the trained model's result for standardization parameters
-    result = model.result
-    Mx = result.Mx  # Mean for standardization
-    Wx = result.Wx  # Standard deviation for standardization
-    
-    # Simulate new data for nowcasting (in practice, this would be real-time data)
-    # Use the last few periods of training data as "new" data
-    n_new_periods = 5
-    X_new_raw = df_processed.iloc[-n_new_periods:].values
-    
-    # Option 1: Update scaler with new data (fit_transform on update batch)
-    # Remove date column if present
-    X_new_df = pd.DataFrame(X_new_raw, columns=df_processed.columns, index=df_processed.index[-n_new_periods:])
-    X_new_df_for_preprocessing = X_new_df.drop(columns=['date']) if 'date' in X_new_df.columns else X_new_df
-    
-    update_pipeline = preprocessing_pipeline.clone().fit(X_new_df_for_preprocessing)
-    X_new_std = update_pipeline.transform(X_new_df_for_preprocessing)
-    update_scaler = _get_fitted_scaler(update_pipeline, X_new_df_for_preprocessing)
-    
-    # Handle any NaN values (missing data in new observations)
-    X_new_std = np.where(np.isfinite(X_new_std), X_new_std, np.nan)
-    
-    print(f"   New data shape: {X_new_std.shape}")
-    print(f"   Standardized new data (first row): {X_new_std[0, :5]}")
-    
-    # Update model state with new standardized data, then predict
-    # Pattern: model.update(X_std).predict(horizon=1)
-    X_nowcast, Z_nowcast = model.update(X_new_std).predict(horizon=1)
-    
-    # Extract nowcast for target series
-    target_idx = selected_cols.index(target_col)
-    nowcast_value = X_nowcast[0, target_idx]
-    
-    print(f"   Nowcast value for {target_col}: {nowcast_value:.6f}")
-    print(f"   Nowcast uses VAR(1) factor dynamics")
-    try:
-        restored_now = scaler.inverse_transform(scaler.transform(X_nowcast))
-        assert np.allclose(restored_now, X_nowcast, atol=1e-6)
-        print("   ✔ Inverse-transform check passed (nowcast, training scaler)")
-    except Exception as inv_err:
-        print(f"   ⚠ Inverse-transform check failed (nowcast): {inv_err}")
-    try:
-        restored_now_upd = update_scaler.inverse_transform(update_scaler.transform(X_nowcast))
-        assert np.allclose(restored_now_upd, X_nowcast, atol=1e-6)
-        print("   ✔ Inverse-transform check passed (nowcast, update scaler)")
-    except Exception as inv_err:
-        print(f"   ⚠ Inverse-transform check failed (nowcast, update scaler): {inv_err}")
-    
-    # Alternative: Update and predict separately
-    model.update(X_new_std)
-    X_nowcast2, Z_nowcast2 = model.predict(horizon=1)
-    nowcast_value2 = X_nowcast2[0, target_idx]
-    print(f"   Alternative pattern (separate calls): {nowcast_value2:.6f}")
-    try:
-        restored_now2 = scaler.inverse_transform(scaler.transform(X_nowcast2))
-        assert np.allclose(restored_now2, X_nowcast2, atol=1e-6)
-        print("   ✔ Inverse-transform check passed (nowcast alt, training scaler)")
-    except Exception as inv_err:
-        print(f"   ⚠ Inverse-transform check failed (nowcast alt): {inv_err}")
-    try:
-        restored_now2_upd = update_scaler.inverse_transform(update_scaler.transform(X_nowcast2))
-        assert np.allclose(restored_now2_upd, X_nowcast2, atol=1e-6)
-        print("   ✔ Inverse-transform check passed (nowcast alt, update scaler)")
-    except Exception as inv_err:
-        print(f"   ⚠ Inverse-transform check failed (nowcast alt, update scaler): {inv_err}")
-    
-except (ValueError, AttributeError, IndexError) as e:
-    print(f"   Nowcasting failed: {e}")
-    print("   Note: Ensure model is trained and data is properly standardized")
-
-# ============================================================================
-# Step 8: Summary
+# Step 7: Summary
 # ============================================================================
 print("\n" + "=" * 80)
 print("Tutorial Summary")
 print("=" * 80)
 print(f"✅ Data loaded: {df.shape[0]} rows, {len(selected_cols)} series")
-print(f"✅ Model trained: {len(series_configs)} series, {config.blocks[DEFAULT_BLOCK_NAME]['factors']} factors, VAR(1) dynamics")
+print(f"✅ Model trained: {len(selected_cols)} series, {config.blocks['Block_Global']['num_factors']} factors, VAR(1) dynamics")
 if X_forecast is not None:
     print(f"✅ Predictions generated: {X_forecast.shape[0]} periods ahead")
 else:
     print(f"⚠️  Predictions: Failed (see error message above)")
-print(f"✅ Nowcasting pattern: model.update(X_std).predict(horizon=1)")
-print(f"✅ Target series: {target_col}")
+print(f"✅ Target series: {', '.join(target_cols)} ({len(target_cols)} targets)")
 print("=" * 80)
